#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "image.h"

void print_usage(const char *argv0);

int main(int argc, char **argv)
{
	int i, mode = 0;
	int text = 0;
	int renibble = 0;
	char *outfname = 0;
	char *infiles[256];
	int num_infiles = 0;
	struct image img, tmpimg;
	FILE *out = stdout;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(argv[i][2] == 0) {
				switch(argv[i][1]) {
				case 'p':
					mode = 0;
					break;

				case 'c':
					mode = 1;
					break;

				case 'i':
					mode = 2;
					break;

				case 't':
					text = 1;
					break;

				case 'n':
					renibble = 1;
					break;

				case 'o':
					if(!argv[++i]) {
						fprintf(stderr, "%s must be followed by a filename\n", argv[i - 1]);
						return 1;
					}
					outfname = argv[i];
					break;

				case 'h':
					print_usage(argv[0]);
					return 0;

				default:
					fprintf(stderr, "invalid option: %s\n", argv[i]);
					print_usage(argv[0]);
					return 1;
				}
			} else {
				fprintf(stderr, "invalid option: %s\n", argv[i]);
				print_usage(argv[0]);
				return 1;
			}
		} else {
			infiles[num_infiles++] = argv[i];
		}
	}

	if(!num_infiles) {
		fprintf(stderr, "pass the filename of a PNG file\n");
		return 1;
	}
	if(load_image(&img, infiles[0]) == -1) {
		fprintf(stderr, "failed to load PNG file: %s\n", infiles[0]);
		return 1;
	}

	for(i=1; i<num_infiles; i++) {
		if(load_image(&tmpimg, infiles[i]) == -1) {
			fprintf(stderr, "failed to load PNG file: %s\n", infiles[i]);
			return 1;
		}
		if(tmpimg.width != img.width || tmpimg.height != img.height) {
			fprintf(stderr, "size mismatch: first image (%s) is %dx%d, %s is %dx%d\n",
					infiles[0], img.width, img.height, infiles[i], tmpimg.width, tmpimg.height);
			return 1;
		}
		if(tmpimg.bpp != img.bpp) {
			fprintf(stderr, "bpp mismatch: first image (%s) is %d bpp, %s is %d bpp\n",
					infiles[0], img.bpp, infiles[i], img.bpp);
			return 1;
		}

		overlay_key(&tmpimg, 0, &img);
	}

	if(img.bpp == 4 && renibble) {
		unsigned char *ptr = img.pixels;
		for(i=0; i<img.width * img.height; i++) {
			unsigned char p = *ptr;
			*ptr++ = (p << 4) | (p >> 4);
		}
	}

	if(outfname) {
		if(!(out = fopen(outfname, "wb"))) {
			fprintf(stderr, "failed to open output file: %s: %s\n", outfname, strerror(errno));
			return 1;
		}
	}

	switch(mode) {
	case 0:
		fwrite(img.pixels, 1, img.scansz * img.height, out);
		break;

	case 1:
		if(text) {
			for(i=0; i<img.cmap_ncolors; i++) {
				printf("%d %d %d\n", img.cmap[i].r, img.cmap[i].g, img.cmap[i].b);
			}
		} else {
			/*fwrite(img.cmap, sizeof img.cmap[0], img.cmap_ncolors, out);*/
			fwrite(img.cmap, sizeof img.cmap[0], 1 << img.bpp, out);
		}
		break;

	case 2:
		printf("size: %dx%d\n", img.width, img.height);
		printf("bit depth: %d\n", img.bpp);
		printf("scanline size: %d bytes\n", img.scansz);
		if(img.cmap_ncolors > 0) {
			printf("colormap entries: %d\n", img.cmap_ncolors);
		} else {
			printf("color channels: %d\n", img.nchan);
		}
		break;
	}

	fclose(out);
	return 0;
}

void print_usage(const char *argv0)
{
	printf("Usage: %s [options] <input file>\n", argv0);
	printf("Options:\n");
	printf(" -o <output file>: specify output file (default: stdout)\n");
	printf(" -p: dump pixels (default)\n");
	printf(" -c: dump colormap (palette) entries\n");
	printf(" -i: print image information\n");
	printf(" -t: dump as text\n");
	printf(" -n: swap the order of nibbles (for 4bpp)\n");
	printf(" -h: print usage and exit\n");
}
